@@ -4,13 +4,13 @@
 #' with constraints Fx+g >= 0 using the Harmonic Hamiltonian Monte Carlo sampler 
 #' (Harmonic-HMC).
 #'
-#' @param n number of samples after burn-in.
+#' @param nSample number of samples after burn-in.
 #' @param burnin number of burn-in samples (default = 0).
 #' @param mean a d-dimensional mean vector.
 #' @param choleskyFactor upper triangular matrix R from Cholesky decomposition of 
 #' precision or covariance matrix into R^TR. 
-#' @param F F matrix (k-by-d matrix where k is the number of linear constraints). 
-#' @param g g vector (k-dimensional). 
+#' @param constrainDirec the k-by-d F matrix (k is the number of linear constraints). 
+#' @param constrainBound the k-dimensional g vector. 
 #' @param init a d-dimensional vector of the initial value. `init` must satisfy all constraints. 
 #' @param time HMC integration time for each iteration. Can either be
 #' a scalar value for a fixed time across all samples, or a length 2 vector of a
@@ -18,13 +18,14 @@
 #' from for each iteration.
 #' @param precFlg logical. whether `choleskyFactor` is from precision
 #' (`TRUE`) or covariance matrix (`FALSE`).
-#' @param diagnosticMode logical. `TRUE` for also returning the bounce distances
-#' for each sample.
+#' @param seed random seed (default = 1).
+#' @param extraOutputs vector of strings. "numBounces" and/or "bounceDistances" 
+#' can be requested, with the latter containing the distances in-between bounces
+#' for each sample and hence incurring significant computational and memory costs.
 #'
-#' @return List of
-#' `samples`: (n + burnin) x d matrix of samples (including burnin samples) and
-#' `bounceDistances`: list of bounces for each sample (only present if
-#' `diagnosticMode` is `TRUE`).
+#' @return 
+#' `samples`: nSample-by-d matrix of samples 
+#' or, if `extraOutputs` is non-empty, a list of `samples` and the extra outputs.
 #' @export
 #'
 #' @examples
@@ -34,49 +35,55 @@
 #' Sigma <- t(A) %*% A
 #' R <- cholesky(Sigma)
 #' mu <- rep(0, d)
-#' F <- diag(d)
-#' g <- rep(0,d)
+#' constrainDirec <- diag(d)
+#' constrainBound <- rep(0,d)
 #' initial <- rep(1, d)
-#' results <- harmonicHMC(1000, 1000, mu, R, F, g, initial, precFlg = FALSE)
+#' results <- harmonicHMC(1000, 1000, mu, R, constrainDirec, constrainBound, initial, precFlg = FALSE)
 #' @references
 #' \insertRef{pakman2014exact}{hdtg}
-harmonicHMC <- function(n,
+harmonicHMC <- function(nSample,
                         burnin = 0,
                         mean,
                         choleskyFactor,
-                        F,
-                        g,
+                        constrainDirec,
+                        constrainBound,
                         init,
                         time = c(pi / 8, pi / 2),
                         precFlg,
-                        diagnosticMode = FALSE) {
+                        seed = NULL,
+                        extraOutputs = c()) {
   if (length(time) == 1) {
     time[2] <- time[1]
   }
   if (time[2] < time[1]) {
     stop("Upper bound for integration time must be greater than lower bound.")
   }
-  if (sum(F %*% init + g > 0) < length(g)) {
+  if (sum(constrainDirec %*% init + constrainBound > 0) < length(constrainBound)) {
     stop("Initial value x does not satisfy Fx + g >=0")
   }
-  samples <- matrix(nrow = n + burnin, ncol = ncol(F))
+  samples <- matrix(nrow = nSample, ncol = ncol(constrainDirec))
   randomBounceTime <-
     ifelse(length(time) == 2, TRUE, FALSE)
+  numBounces <- vector(mode = "integer", length = nSample)
+  diagnosticMode <- "bounceDistances" %in% extraOutputs
   bounceDistances <- vector(mode = "list",
-                            length = ifelse(diagnosticMode, n + burnin, 0))
-  whitenedConstraints <- applyWhitenTransform(F,
-                                              g,
+                            length = ifelse(diagnosticMode, nSample, 0))
+  whitenedConstraints <- applyWhitenTransform(constrainDirec,
+                                              constrainBound,
                                               choleskyFactor,
                                               mean,
                                               precFlg)
   position <- whitenPosition(init,
-                             F,
-                             g,
+                             constrainDirec,
+                             constrainBound,
                              choleskyFactor,
                              mean,
                              precFlg)
-  for (i in 1:(n + burnin)) {
-    momentum <- rnorm(ncol(F))
+  if (!is.null(seed)) {
+    set.seed(seed)
+  }
+  for (i in 1:(nSample + burnin)) {
+    momentum <- rnorm(ncol(constrainDirec))
     results <- simulateWhitenedDynamics(
       position,
       momentum,
@@ -87,17 +94,24 @@ harmonicHMC <- function(n,
       diagnosticMode
     )
     position <- results$position
-    samples[i,] <- unwhitenPosition(position,
-                                    choleskyFactor,
-                                    mean,
-                                    precFlg)
-    if (diagnosticMode) {
-      bounceDistances[[i]] <- results$bounceDistances
+    if (i > burnin) {
+      samples[i - burnin, ] <- unwhitenPosition(position,
+                                                choleskyFactor,
+                                                mean,
+                                                precFlg)
+      numBounces[i - burnin] <- results$numBounces
+      if ("bounceDistances" %in% extraOutputs) {
+        bounceDistances[[i - burnin]] <- results$bounceDistances
+      }
     }
   }
-  if (diagnosticMode) {
-    return(list("samples" = samples, "bounceDistances" = bounceDistances))
+  if (length(extraOutputs) == 0) {
+    return(samples)
   } else {
-    return(list("samples" = samples))
+    output <- list("samples" = samples)
+    for (quantity in extraOutputs) {
+      output[[quantity]] <- get(quantity)
+    }
+    return(output)
   }
 }
